import { applyMiddleware, compose } from "redux";
import { ARENA_INIT_AUDIENCE_SAGA } from "./actionTypes";
import createSagaMiddleware, { END } from "redux-saga";
import { getArenaInitState, arenaReducer } from "./reducers";
import { createEnhancedStore } from "./enhancedRedux";
import rootSaga from "./sagas";

/**
 * This is an initialization function,
 * you can pass some of the initialization of some
 * initialization reducer, state, saga or some other redux middleware,
 * such as redux-thunk
 * If you pass in a custom saga, it will use fork to run once and
 * keep the running task information in the root of the arena
 * @export
 * @param {any} [reducers={}]  
 * @param {any} [initialStates={}] 
 * @param {any} saga 
 * @param {any} enhencers 
 * @returns
 */
export function createArenaStore() {
  var reducers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var initialStates = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var enhencers = arguments[2];
  var sagaOptions = arguments[3];

  var sagaMiddleware = createSagaMiddleware(sagaOptions);
  var store = createEnhancedStore(Object.assign({
    arena: arenaReducer
  }, reducers), Object.assign({
    arena: getArenaInitState()
  }, initialStates), enhencers ? compose.apply(undefined, [applyMiddleware(sagaMiddleware)].concat(enhencers)) : applyMiddleware(sagaMiddleware));
  sagaMiddleware.run(rootSaga, { store: store });
  store.close = function () {
    return store.dispatch(END);
  };
  store.runSaga = function (saga) {
    return store.dispatch({
      type: ARENA_INIT_AUDIENCE_SAGA,
      saga: saga
    });
  };
  return store;
}