"use strict";

exports.__esModule = true;
exports.default = createEnhancedStore;

var _redux = require("redux");

function storeEnhancer(store, reducers) {
  var _currentReducers = reducers;
  var handler = {
    get: function get(target, name) {
      if (name === "addReducer") {
        return function (_ref) {
          var _Object$assign;

          var reducerKey = _ref.reducerKey,
              reducer = _ref.reducer;

          var allStates = target.getState();
          if (allStates[reducerKey] != null) return false;
          _currentReducers = Object.assign({}, _currentReducers, (_Object$assign = {}, _Object$assign[reducerKey] = reducer, _Object$assign));
          target.replaceReducer((0, _redux.combineReducers)(_currentReducers));
          return true;
        };
      }
      if (name === "removeReducer") {
        return function (reducerKey) {
          if (reducerKey == null) return false;
          var newReducers = Object.assign({}, _currentReducers);
          var allStates = target.getState();
          delete newReducers[reducerKey];
          _currentReducers = newReducers;
          delete allStates[reducerKey];
          target.replaceReducer((0, _redux.combineReducers)(newReducers));
          return true;
        };
      }
      if (name === "replaceReducer") {
        return function (_ref2) {
          var _Object$assign2;

          var reducerKey = _ref2.reducerKey,
              reducer = _ref2.reducer;

          if (reducerKey == null) throw new Error("reducerKey can not be null.");
          var allStates = target.getState();
          if (_currentReducers[reducerKey] == null) throw new Error("reducer for key [" + reducerKey + "] doesn't exsit.");
          _currentReducers = Object.assign({}, _currentReducers, (_Object$assign2 = {}, _Object$assign2[reducerKey] = reducer, _Object$assign2));
          target.replaceReducer((0, _redux.combineReducers)(_currentReducers));
          return reducerKey;
        };
      }
      if (name === "removeAndAddReducer") {
        return function (_ref3) {
          var reducerKeyRemoved = _ref3.reducerKeyRemoved,
              reducerKeyAdded = _ref3.reducerKeyAdded,
              reducer = _ref3.reducer;

          if (reducerKeyRemoved == null || reducerKeyAdded == null) throw new Error("reducerKeyRemoved or reducerKeyAdded can not be null.");
          if (_currentReducers[reducerKeyRemoved] == null) throw new Error("reducer of key [" + reducerKeyRemoved + "] doesn't exsit.");
          var newReducers = Object.assign({}, _currentReducers);
          var allStates = target.getState();
          delete allStates[reducerKeyRemoved];
          delete newReducers[reducerKeyRemoved];
          if (newReducers[reducerKeyAdded] != null) throw new Error("reducer of key [" + reducerKeyAdded + "] already exsit.");
          newReducers[reducerKeyAdded] = reducer;
          _currentReducers = newReducers;
          target.replaceReducer((0, _redux.combineReducers)(newReducers));
          return reducerKeyAdded;
        };
      }
      return target[name];
    }
  };
  return new Proxy(store, handler);
} /**
   * Create redux-arena proxy store
   */
function createEnhancedStore(reducers, initialState, enhencer) {
  var store = (0, _redux.createStore)((0, _redux.combineReducers)(reducers), initialState, enhencer);
  return storeEnhancer(store, reducers);
}