"use strict";

exports.__esModule = true;
exports.createArenaStore = createArenaStore;

var _redux = require("redux");

var _actionTypes = require("./actionTypes");

var _reduxSaga = require("redux-saga");

var _reduxSaga2 = _interopRequireDefault(_reduxSaga);

var _reducers = require("./reducers");

var _enhancedRedux = require("./enhancedRedux");

var _sagas = require("./sagas");

var _sagas2 = _interopRequireDefault(_sagas);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * This is an initialization function,
 * you can pass some of the initialization of some
 * initialization reducer, state, saga or some other redux middleware,
 * such as redux-thunk
 * If you pass in a custom saga, it will use fork to run once and
 * keep the running task information in the root of the arena
 * @export
 * @param {any} [reducers={}]  
 * @param {any} [initialStates={}] 
 * @param {any} saga 
 * @param {any} enhencers 
 * @returns
 */
function createArenaStore() {
  var reducers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var initialStates = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var enhencers = arguments[2];
  var sagaOptions = arguments[3];

  var sagaMiddleware = (0, _reduxSaga2.default)(sagaOptions);
  var store = (0, _enhancedRedux.createEnhancedStore)(Object.assign({
    arena: _reducers.arenaReducer
  }, reducers), Object.assign({
    arena: (0, _reducers.getArenaInitState)()
  }, initialStates), enhencers ? _redux.compose.apply(undefined, [(0, _redux.applyMiddleware)(sagaMiddleware)].concat(enhencers)) : (0, _redux.applyMiddleware)(sagaMiddleware));
  sagaMiddleware.run(_sagas2.default, { store: store });
  store.close = function () {
    return store.dispatch(_reduxSaga.END);
  };
  store.runSaga = function (saga) {
    return store.dispatch({
      type: _actionTypes.ARENA_INIT_AUDIENCE_SAGA,
      saga: saga
    });
  };
  return store;
}