"use strict";

exports.__esModule = true;
exports.applySceneBundle = applySceneBundle;
exports.applyAsyncSceneBundle = applyAsyncSceneBundle;

var _actionTypes = require("../actionTypes");

var _effects = require("redux-saga/effects");

var _reactRedux = require("react-redux");

var _enhancedRedux = require("../enhancedRedux");

var _sceneReduxSaga = require("./sceneReduxSaga");

var _marked = /*#__PURE__*/regeneratorRuntime.mark(applySceneBundle),
    _marked2 = /*#__PURE__*/regeneratorRuntime.mark(applyAsyncSceneBundle);

/**
 * Scene of the synchronous load function, it does the following.
 * 1. load, add, replace reducer.
 * 2. Set the state of the corresponding scene.
 * 3. Run the corresponding scene of the saga, cancel the last scene of the task.
 * 4. Connect redux according to the incoming mapStateToProps and actions.
 * 
 * @param {any} { arenaReducerDict, sceneBundle } 
 */
function applySceneBundle(_ref) {
  var isInitial = _ref.isInitial,
      parentArenaReducerDict = _ref.parentArenaReducerDict,
      sceneBundle = _ref.sceneBundle,
      notifyAction = _ref.notifyAction;

  var arenaCurtainReducerKey, _ref2, curSceneBundle, reduxInfo, OldPlayingScene, arenaReducerDict, newReduxInfo, propsPicker, PlayingScene, displayName, newArenaState;

  return regeneratorRuntime.wrap(function applySceneBundle$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          arenaCurtainReducerKey = parentArenaReducerDict._arenaCurtain.reducerKey;
          _context.next = 3;
          return (0, _effects.select)(function (state) {
            return state[arenaCurtainReducerKey];
          });

        case 3:
          _ref2 = _context.sent;
          curSceneBundle = _ref2.curSceneBundle;
          reduxInfo = _ref2.reduxInfo;
          OldPlayingScene = _ref2.PlayingScene;
          arenaReducerDict = _ref2.arenaReducerDict;
          newReduxInfo = void 0;

          if (!isInitial) {
            _context.next = 14;
            break;
          }

          return _context.delegateYield((0, _sceneReduxSaga.sceneApplyRedux)({
            parentArenaReducerDict: parentArenaReducerDict,
            state: sceneBundle.state,
            saga: sceneBundle.saga,
            actions: sceneBundle.actions,
            reducer: sceneBundle.reducer,
            options: sceneBundle.options || {}
          }), "t0", 11);

        case 11:
          newReduxInfo = _context.t0;
          _context.next = 16;
          break;

        case 14:
          return _context.delegateYield((0, _sceneReduxSaga.sceneUpdateRedux)({
            parentArenaReducerDict: parentArenaReducerDict,
            state: sceneBundle.state,
            saga: sceneBundle.saga,
            actions: sceneBundle.actions,
            reducer: sceneBundle.reducer,
            options: sceneBundle.options || {},
            curSceneBundle: curSceneBundle,
            reduxInfo: reduxInfo
          }), "t1", 15);

        case 15:
          newReduxInfo = _context.t1;

        case 16:
          _context.next = 18;
          return (0, _effects.put)({
            type: _actionTypes.ARENA_CURTAIN_SET_STATE,
            _reducerKey: arenaCurtainReducerKey,
            state: {
              reduxInfo: newReduxInfo
            }
          });

        case 18:
          propsPicker = (0, _enhancedRedux.createPropsPicker)(sceneBundle.propsPicker, newReduxInfo);
          PlayingScene = (0, _reactRedux.connect)(propsPicker)(sceneBundle.Component);
          displayName = sceneBundle.Component.displayName || sceneBundle.Component.name || "Unknown";

          PlayingScene.displayName = "SceneConnect({reducerKey:" + newReduxInfo.reducerKey + ",Component:" + displayName + "})";
          newArenaState = {
            PlayingScene: PlayingScene,
            curSceneBundle: sceneBundle
          };
          _context.next = 25;
          return (0, _effects.put)({
            type: _actionTypes.ARENA_CURTAIN_SET_STATE,
            _reducerKey: arenaCurtainReducerKey,
            state: newArenaState
          });

        case 25:
        case "end":
          return _context.stop();
      }
    }
  }, _marked, this);
}

/**
 * The asynchronous loading function of the scene, 
 * and finally the synchronous load function
 * 
 * @param {any} { arenaReducerDict, asyncSceneBundle } 
 * @returns 
 */
function applyAsyncSceneBundle(_ref3) {
  var isInitial = _ref3.isInitial,
      parentArenaReducerDict = _ref3.parentArenaReducerDict,
      asyncSceneBundle = _ref3.asyncSceneBundle,
      notifyAction = _ref3.notifyAction;
  var sceneBundle;
  return regeneratorRuntime.wrap(function applyAsyncSceneBundle$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          sceneBundle = void 0;
          _context2.prev = 1;
          _context2.next = 4;
          return asyncSceneBundle;

        case 4:
          sceneBundle = _context2.sent;

        case 5:
          _context2.prev = 5;
          _context2.next = 8;
          return (0, _effects.cancelled)();

        case 8:
          if (!_context2.sent) {
            _context2.next = 9;
            break;
          }

        case 9:
          return _context2.finish(5);

        case 10:
          sceneBundle = sceneBundle.default ? sceneBundle.default : sceneBundle;
          return _context2.delegateYield(applySceneBundle({
            isInitial: isInitial,
            parentArenaReducerDict: parentArenaReducerDict,
            sceneBundle: sceneBundle,
            notifyAction: notifyAction
          }), "t0", 12);

        case 12:
          return _context2.abrupt("return", true);

        case 13:
        case "end":
          return _context2.stop();
      }
    }
  }, _marked2, this, [[1,, 5, 10]]);
}