"use strict";

exports.__esModule = true;
exports.sceneApplyRedux = sceneApplyRedux;
exports.sceneUpdateRedux = sceneUpdateRedux;

var _actionTypes = require("../actionTypes");

var _actionTypes2 = require("../../actionTypes");

var _effects = require("redux-saga/effects");

var _redux = require("redux");

var _enhancedRedux = require("../enhancedRedux");

var _reducers = require("../reducers");

var _utils = require("../../utils");

var _marked = /*#__PURE__*/regeneratorRuntime.mark(forkSagaWithContext),
    _marked2 = /*#__PURE__*/regeneratorRuntime.mark(sceneApplyRedux),
    _marked3 = /*#__PURE__*/regeneratorRuntime.mark(sceneUpdateRedux);

var defaultActions = {
  setState: function setState(state) {
    return { type: _actionTypes2.ARENA_SCENE_SET_STATE, state: state };
  }
};

function calcNewReduxInfo(reduxInfo, newReduxInfo, dispatch, isSceneActions) {
  var connectedActions = reduxInfo.connectedActions;
  var newArenaReducerDict = reduxInfo.arenaReducerDict;
  if (reduxInfo.reducerKey !== newReduxInfo.reducerKey || reduxInfo.vReducerKey !== newReduxInfo.vReducerKey || reduxInfo.parentArenaReducerDict !== newReduxInfo.parentArenaReducerDict || reduxInfo.actions !== newReduxInfo.actions) {
    var _Object$assign;

    //calc actions
    if (reduxInfo.actions !== newReduxInfo.actions || connectedActions == null) {
      if (isSceneActions === false) {
        connectedActions = (0, _redux.bindActionCreators)(newReduxInfo.actions || defaultActions, dispatch);
      } else {
        connectedActions = (0, _enhancedRedux.bindArenaActionCreators)(newReduxInfo.actions || defaultActions, dispatch, newReduxInfo.reducerKey);
      }
    }
    //calc arena reducer dict
    var item = {
      reducerKey: newReduxInfo.reducerKey,
      vReducerKey: newReduxInfo.vReducerKey,
      actions: connectedActions
    };
    newArenaReducerDict = Object.assign(newReduxInfo.parentArenaReducerDict, (_Object$assign = {}, _Object$assign[newReduxInfo.reducerKey] = item, _Object$assign._arenaScene = item, _Object$assign));
    if (newReduxInfo.vReducerKey != null) newArenaReducerDict[newReduxInfo.vReducerKey] = item;
  }
  return Object.assign({}, newReduxInfo, {
    connectedActions: connectedActions,
    arenaReducerDict: newArenaReducerDict
  });
}

function forkSagaWithContext(saga, ctx) {
  return regeneratorRuntime.wrap(function forkSagaWithContext$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return (0, _effects.setContext)(ctx);

        case 2:
          _context.next = 4;
          return (0, _effects.fork)(saga);

        case 4:
        case "end":
          return _context.stop();
      }
    }
  }, _marked, this);
}

function sceneApplyRedux(_ref) {
  var parentArenaReducerDict = _ref.parentArenaReducerDict,
      state = _ref.state,
      saga = _ref.saga,
      actions = _ref.actions,
      reducer = _ref.reducer,
      options = _ref.options;
  var arenaStore, reducerFactory, newReducerKey, newReduxInfo;
  return regeneratorRuntime.wrap(function sceneApplyRedux$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return (0, _effects.getContext)("store");

        case 2:
          arenaStore = _context2.sent;
          reducerFactory = options.isSceneReducer === false ? function (bindingReducerKey) {
            return (0, _reducers.createSceneReducer)(reducer, bindingReducerKey, state, parentArenaReducerDict);
          } : function (bindingReducerKey) {
            return (0, _reducers.createSceneReducer)(reducer && (0, _reducers.sceneReducerWrapper)(reducer), bindingReducerKey, state, parentArenaReducerDict);
          };
          newReducerKey = (0, _utils.sceneAddReducer)(arenaStore, options.reducerKey, reducerFactory, state);
          newReduxInfo = calcNewReduxInfo({}, {
            reducerKey: newReducerKey,
            vReducerKey: options.vReducerKey,
            parentArenaReducerDict: parentArenaReducerDict,
            actions: actions
          }, arenaStore.dispatch, options.isSceneActions);

          if (!saga) {
            _context2.next = 10;
            break;
          }

          _context2.next = 9;
          return (0, _effects.fork)(forkSagaWithContext, saga, {
            arenaReducerDict: newReduxInfo.arenaReducerDict
          });

        case 9:
          newReduxInfo.sagaTask = _context2.sent;

        case 10:
          return _context2.abrupt("return", newReduxInfo);

        case 11:
        case "end":
          return _context2.stop();
      }
    }
  }, _marked2, this);
}

function sceneUpdateRedux(_ref2) {
  var parentArenaReducerDict = _ref2.parentArenaReducerDict,
      state = _ref2.state,
      saga = _ref2.saga,
      actions = _ref2.actions,
      reducer = _ref2.reducer,
      options = _ref2.options,
      curSceneBundle = _ref2.curSceneBundle,
      reduxInfo = _ref2.reduxInfo;
  var newReducerKey, arenaStore, reducerFactory, oldState, newReduxInfo;
  return regeneratorRuntime.wrap(function sceneUpdateRedux$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          newReducerKey = reduxInfo.reducerKey;
          _context3.next = 3;
          return (0, _effects.getContext)("store");

        case 3:
          arenaStore = _context3.sent;
          _context3.prev = 4;

          if (!(saga !== curSceneBundle.saga)) {
            _context3.next = 9;
            break;
          }

          if (!reduxInfo.sagaTask) {
            _context3.next = 9;
            break;
          }

          _context3.next = 9;
          return (0, _effects.cancel)(reduxInfo.sagaTask);

        case 9:
          _context3.prev = 9;
          _context3.next = 12;
          return (0, _effects.cancelled)();

        case 12:
          if (!_context3.sent) {
            _context3.next = 13;
            break;
          }

        case 13:
          return _context3.finish(9);

        case 14:
          reducerFactory = options.isSceneReducer === false ? function (bindingReducerKey) {
            return (0, _reducers.createSceneReducer)(reducer, bindingReducerKey, state);
          } : function (bindingReducerKey) {
            return (0, _reducers.createSceneReducer)(reducer && (0, _reducers.sceneReducerWrapper)(reducer), bindingReducerKey, state);
          };

          if (!(options.reducerKey == null || options.reducerKey === reduxInfo.reducerKey)) {
            _context3.next = 19;
            break;
          }

          if (reducer !== curSceneBundle.reducer) {
            newReducerKey = (0, _utils.sceneReplaceReducer)(arenaStore, reduxInfo.reducerKey, reducerFactory, state === curSceneBundle.state ? null : state);
          } else if (state !== curSceneBundle.state) {
            arenaStore.dispatch({
              type: _actionTypes.ARENA_SCENE_REPLACE_STATE,
              _sceneReducerKey: newReducerKey,
              state: state
            });
          }
          _context3.next = 27;
          break;

        case 19:
          if (!(options.reducerKey !== reduxInfo.reducerKey)) {
            _context3.next = 26;
            break;
          }

          _context3.next = 22;
          return (0, _effects.select)(function (state) {
            return state[reduxInfo.reducerKey];
          });

        case 22:
          oldState = _context3.sent;

          newReducerKey = (0, _utils.sceneRmAndAddReducer)(arenaStore, reduxInfo.reducerKey, options.reducerKey, reducerFactory, state === curSceneBundle.state ? oldState : state);
          _context3.next = 27;
          break;

        case 26:
          if (state !== curSceneBundle.state) {
            arenaStore.dispatch({
              type: _actionTypes.ARENA_SCENE_REPLACE_STATE,
              _sceneReducerKey: newReducerKey,
              state: state
            });
          }

        case 27:
          newReduxInfo = calcNewReduxInfo(reduxInfo, {
            reducerKey: newReducerKey,
            vReducerKey: options.vReducerKey,
            parentArenaReducerDict: parentArenaReducerDict,
            actions: actions
          }, arenaStore.dispatch, options.isSceneActions);

          if (!saga) {
            _context3.next = 33;
            break;
          }

          if (!(saga !== curSceneBundle.saga || newReducerKey !== reduxInfo.reducerKey || reduxInfo.sagaTask == null || reduxInfo.sagaTask.isCancelled())) {
            _context3.next = 33;
            break;
          }

          _context3.next = 32;
          return (0, _effects.fork)(forkSagaWithContext, saga, {
            arenaReducerDict: newReduxInfo.arenaReducerDict
          });

        case 32:
          newReduxInfo.sagaTask = _context3.sent;

        case 33:
          return _context3.abrupt("return", newReduxInfo);

        case 34:
        case "end":
          return _context3.stop();
      }
    }
  }, _marked3, this, [[4,, 9, 14]]);
}