"use strict";

exports.__esModule = true;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _actionTypes = require("../../core/actionTypes");

var _reducers = require("../../core/reducers");

var _utils = require("../../utils");

var _SceneBundle = require("../SceneBundle");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ArenaScene = function (_Component) {
  _inherits(ArenaScene, _Component);

  function ArenaScene() {
    _classCallCheck(this, ArenaScene);

    return _possibleConstructorReturn(this, _Component.apply(this, arguments));
  }

  ArenaScene.prototype.componentWillMount = function componentWillMount() {
    var _this2 = this;

    var reducerKey = (0, _utils.curtainAddReducer)(this.context.store, this.props.reducerKey, _reducers.createCurtainReducer);
    var _props = this.props,
        asyncSceneBundle = _props.asyncSceneBundle,
        sceneBundle = _props.sceneBundle,
        sceneProps = _props.sceneProps,
        isNotifyOn = _props.isNotifyOn,
        notifyData = _props.notifyData;

    var arenaReducerDict = (0, _utils.calcCurtainReducerDict)(this.context.arenaReducerDict, reducerKey, this.props.vReducerKey);
    var wrappedSceneBundle = (0, _SceneBundle.arenaCurtainConnect)(arenaReducerDict);
    var sceneBundleElement = _react2.default.createElement(wrappedSceneBundle, {
      asyncSceneBundle: asyncSceneBundle,
      sceneBundle: sceneBundle,
      sceneProps: sceneProps,
      isNotifyOn: isNotifyOn,
      notifyData: notifyData
    });
    this.setState({
      arenaReducerDict: arenaReducerDict,
      wrappedSceneBundle: wrappedSceneBundle,
      sceneBundleElement: sceneBundleElement,
      sagaTaskPromise: new Promise(function (resolve) {
        return _this2.context.store.dispatch({
          type: _actionTypes.ARENA_CURTAIN_INIT_SAGA,
          reducerKey: reducerKey,
          setSagaTask: resolve
        });
      })
    });
  };

  ArenaScene.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps, nextContext) {
    var refreshFlag = false;
    var state = Object.assign({}, this.state);
    var reducerKey = nextProps.reducerKey,
        vReducerKey = nextProps.vReducerKey,
        asyncSceneBundle = nextProps.asyncSceneBundle,
        sceneBundle = nextProps.sceneBundle,
        sceneProps = nextProps.sceneProps,
        isNotifyOn = nextProps.isNotifyOn,
        notifyData = nextProps.notifyData;

    var newReducerKey = state.arenaReducerDict._arenaCurtain.reducerKey;
    if (reducerKey != null && reducerKey !== state.arenaReducerDict._arenaCurtain.reducerKey) {
      refreshFlag = true;
      nextContext.store.dispatch({
        type: _actionTypes.ARENA_CURTAIN_CLEAR_REDUX,
        reducerKey: state.arenaReducerDict._arenaCurtain.reducerKey,
        sagaTaskPromise: state.sagaTaskPromise
      });
      newReducerKey = (0, _utils.curtainAddReducer)(nextContext.store, reducerKey, _reducers.createCurtainReducer);
      state.sagaTaskPromise = new Promise(function (resolve) {
        return nextContext.store.dispatch({
          type: _actionTypes.ARENA_CURTAIN_INIT_SAGA,
          reducerKey: newReducerKey,
          setSagaTask: resolve
        });
      });
    }
    if (nextContext.arenaReducerDict !== this.context.arenaReducerDict || reducerKey !== this.props.reducerKey || vReducerKey !== this.props.vReducerKey || refreshFlag === true) {
      refreshFlag = true;
      state.arenaReducerDict = (0, _utils.calcCurtainReducerDict)(nextContext.arenaReducerDict, newReducerKey, nextProps.vReducerKey);
      state.wrappedSceneBundle = (0, _SceneBundle.arenaCurtainConnect)(state.arenaReducerDict);
    }
    if (asyncSceneBundle !== this.props.asyncSceneBundle || sceneBundle !== this.props.sceneBundle || sceneProps !== this.props.sceneBundle || notifyData !== this.props.notifyData || refreshFlag == true) {
      state.sceneBundleElement = _react2.default.createElement(state.wrappedSceneBundle, {
        asyncSceneBundle: asyncSceneBundle,
        sceneBundle: sceneBundle,
        sceneProps: sceneProps,
        isNotifyOn: isNotifyOn,
        notifyData: notifyData
      });
    }
    this.setState(state);
  };

  ArenaScene.prototype.componentWillUnmount = function componentWillUnmount() {
    this.context.store.dispatch({
      type: _actionTypes.ARENA_CURTAIN_CLEAR_REDUX,
      reducerKey: this.state.arenaReducerDict._arenaCurtain.reducerKey,
      sagaTaskPromise: this.state.sagaTaskPromise
    });
  };

  ArenaScene.prototype.render = function render() {
    return this.state.sceneBundleElement;
  };

  return ArenaScene;
}(_react.Component);

ArenaScene.contextTypes = {
  store: _propTypes2.default.any,
  arenaReducerDict: _propTypes2.default.object
};
ArenaScene.propTypes = {
  reducerKey: _propTypes2.default.string,
  vReducerKey: _propTypes2.default.string,
  sceneBundle: _propTypes2.default.object,
  asyncSceneBundle: _propTypes2.default.object,
  sceneProps: _propTypes2.default.object,
  isNotifyOn: _propTypes2.default.bool,
  notifyData: _propTypes2.default.object
};
exports.default = ArenaScene;