"use strict";

exports.__esModule = true;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactMotion = require("react-motion");

var _animationPhase = require("./animationPhase");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ArenaSceneAnimation = function (_Component) {
  _inherits(ArenaSceneAnimation, _Component);

  function ArenaSceneAnimation() {
    _classCallCheck(this, ArenaSceneAnimation);

    return _possibleConstructorReturn(this, _Component.apply(this, arguments));
  }

  ArenaSceneAnimation.prototype.cloneSceneWithNotify = function cloneSceneWithNotify(children, reducerKey) {
    return _react2.default.cloneElement(children, {
      isNotifyOn: true,
      notifyData: Object.assign({}, children.props.notifyData, {
        _toReducerKey: reducerKey
      })
    });
  };

  ArenaSceneAnimation.prototype.componentWillMount = function componentWillMount() {
    var _this2 = this;

    this.setState({
      scenePlay: this.cloneSceneWithNotify(this.props.children, this.props.reducerKey),
      initStyles: this.props.initStyles.map(function (styleObj) {
        return Object.assign({}, styleObj, {
          style: Object.assign({}, styleObj.style, { phase: _animationPhase.LOADING })
        });
      }).concat({
        key: "nextPhase",
        style: { phase: _animationPhase.LOADING }
      }),
      styleCalculator: (0, _utils.buildStyleCalculator)(this.props.styleCalculators, this.props.phase, this.props.nextPhaseCheckers, this.props.isSceneReady, function (phase) {
        return setImmediate(function () {
          return _this2.props.actions.nextPhase(phase);
        });
      })
    });
  };

  ArenaSceneAnimation.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var state = Object.assign({}, state);
    if (nextProps.children !== this.props.children || nextProps.reducerKey !== this.props.reducerKey) {
      state.scenePlay = this.cloneSceneWithNotify(nextProps.children, nextProps.reducerKey);
    }
    if (nextProps.actions !== this.props.actions || nextProps.phase !== this.props.phase || nextProps.styleCalculators !== this.props.styleCalculators || nextProps.nextPhaseCheckers !== this.props.nextPhaseCheckers || nextProps.isSceneReady !== this.props.isSceneReady) {
      state.styleCalculator = (0, _utils.buildStyleCalculator)(nextProps.styleCalculators, nextProps.phase, nextProps.nextPhaseCheckers, nextProps.isSceneReady, function (phase) {
        return setImmediate(function () {
          return nextProps.actions.nextPhase(phase);
        });
      });
    }
    if (nextProps.initStyles !== this.props.initStyles) {
      var nextPhaseStyle = state.initStyles.find(function (style) {
        return style.key === "nextPhase";
      });
      state.initStyles = nextProps.initStyles.concat(nextPhaseStyle);
    }
    this.setState(state);
  };

  ArenaSceneAnimation.prototype.render = function render() {
    var _this3 = this;

    var _props = this.props,
        phase = _props.phase,
        numberToStyle = _props.numberToStyle,
        isSceneReady = _props.isSceneReady;

    return _react2.default.createElement(
      _reactMotion.TransitionMotion,
      {
        defaultStyles: this.state.initStyles,
        willLeave: this.willLeave,
        styles: this.state.styleCalculator
      },
      function (interpolatedStyles) {
        var containerStyle = void 0,
            scenePlayStyle = void 0,
            loadingPlayStyle = void 0,
            animationPhase = void 0;
        interpolatedStyles.forEach(function (styleObj) {
          var key = styleObj.key,
              style = styleObj.style;

          switch (key) {
            case "container":
              containerStyle = style;
              break;
            case "loadingPlay":
              loadingPlayStyle = style;
              break;
            case "scenePlay":
              scenePlayStyle = style;
              break;
            case "nextPhase":
              animationPhase = style.phase;
              break;
          }
        });
        return _react2.default.createElement(
          "div",
          {
            style: numberToStyle("container", containerStyle, animationPhase, isSceneReady)
          },
          animationPhase === _animationPhase.IN || animationPhase === _animationPhase.LEAVING || animationPhase === _animationPhase.OUT ? null : _react2.default.createElement(
            "div",
            {
              key: "loadingPlay",
              style: numberToStyle("loadingPlay", loadingPlayStyle, animationPhase, isSceneReady)
            },
            _this3.props.loadingPlay
          ),
          animationPhase === _animationPhase.OUT ? null : _react2.default.createElement(
            "div",
            {
              key: "scenePlay",
              style: numberToStyle("scenePlay", scenePlayStyle, animationPhase, isSceneReady)
            },
            _this3.state.scenePlay
          )
        );
      }
    );
  };

  return ArenaSceneAnimation;
}(_react.Component);

ArenaSceneAnimation.propTypes = {
  loadingPlay: _propTypes2.default.element.isRequired,
  children: _propTypes2.default.element.isRequired,
  initStyles: _propTypes2.default.array.isRequired,
  styleCalculators: _propTypes2.default.object.isRequired,
  nextPhaseCheckers: _propTypes2.default.object.isRequired,
  numberToStyle: _propTypes2.default.func.isRequired
};
exports.default = ArenaSceneAnimation;